class BaseProxyRouterError(Exception):
    pass


class ProxyAuthenficationError(BaseProxyRouterError):
    pass


class AttributeAuthUserError(BaseProxyRouterError):
    pass


class ConfigTypeError(BaseProxyRouterError):
    pass


class HostNotFoundError(BaseProxyRouterError):
    pass
