from enum import StrEnum
from typing import TypeAlias

PathURL: TypeAlias = str
ProxyPathURL: TypeAlias = str

PathFileConfig: TypeAlias = str
PathDirectoryConfig: TypeAlias = str
PathConfigs: TypeAlias = PathFileConfig | PathDirectoryConfig | list[PathFileConfig]


class AuthType(StrEnum):
    QUERY_PARAMS = "query_params"
    HEADERS = "headers"
